using Microsoft.CodeAnalysis;
using Microsoft.CodeAnalysis.CSharp;
using System.IO;
using System.Reflection;
using System.Runtime.Loader;
using System;

namespace RoslynCore
{
    public static class EmitDemo
    {
        public static void GenerateAssembly()
        {
            const string code = @"using System; 
using System.IO; 

namespace RoslynCore 
{ 
 public static class Helper
 { 
    public static double CalculateCircleArea(double radius) 
    { 
        return radius * radius * Math.PI;
    } 
  } 
}";
            var tree = SyntaxFactory.ParseSyntaxTree(code);
            string fileName="mylib.dll";

            // Detect the file location for the library that defines the 
            // object type
            var systemRefLocation=typeof(object).GetTypeInfo().Assembly.Location;

            // Create a reference to the library
            var systemReference = MetadataReference.CreateFromFile(systemRefLocation);

            var compilation=CSharpCompilation.Create(fileName).
                WithOptions(
                    new CSharpCompilationOptions(OutputKind.DynamicallyLinkedLibrary))
                .AddReferences(systemReference)
                .AddSyntaxTrees(tree);

            var path = Path.Combine(Directory.GetCurrentDirectory(), fileName);
            var compilationResult = compilation.Emit(path);

            if(compilationResult.Success)
            {
                // Load the assembly
                Assembly asm = AssemblyLoadContext.Default.LoadFromAssemblyPath(path);
	
                // Invoke the RoslynCore.Helper.CalculateCircleArea method passing an argument
                double radius = 10;
                object result =  
                       asm.GetType("RoslynCore.Helper").GetMethod("CalculateCircleArea").
                       Invoke(null, new object[] { radius });
                Console.WriteLine(result);
            }
            else
            {
                foreach (Diagnostic codeIssue in compilationResult.Diagnostics)
                {
                    string issue = $"ID: {codeIssue.Id}, Message: {codeIssue.GetMessage()}, Location: {codeIssue.Location.GetLineSpan().ToString  ()}, Severity: {codeIssue.Severity.ToString  ()}";
                    Console.WriteLine(issue);
                }
            }
        }
    }
}